/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.startstopp.prozessvew;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.io.BufferedInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.sys.startstopp.skriptvew.SkriptVerwaltung;
import de.bsvrz.sys.startstopp.skriptvew.StartStoppApp;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_AUSGABE;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_NEUSTART;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVewEnums.E_STARTART;

/**
 * Methode zum Testen der Klasse ProzessVerwaltungTest 
 * @author Dambach Werke GmbH
 */
public class ProzessVerwaltungTest
{
  /**
   * Name der Resource
   */
  private static String m_resource     = "startStopp.xml";
  
  /**
   * Name einer temp.Datei
   */
  private static String m_datei        = "prozessverwaltungTest.xml";
  
  /**
   * Instanz der Klasse SkriptVerwaltung 
   */
  private static SkriptVerwaltung m_sv = null;
  
  /**
   * Instanz der Klasse ProzessVerwaltung
   */
  private static ProzessVerwaltung m_pv;
  
  /**
   * Absender
   */
  private static int m_absender = 1234;

  /**
   * Liste mit den erwarteten Namen der Inkarnationen
   */
  private List<String> m_erwarteteNamen = Arrays.asList ( "Betriebsmeldungsverwaltung",
                                                          "UeberpruefungSystem",
                                                          "Applikation1.1",
                                                          "Applikation1.2",
                                                          "Parametrierung",
                                                          "Applikation1.4",
                                                          "Applikation1.3",
                                                          "Konfiguration",
                                                          "Datenverteiler");

  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    // Testumgebung initialisieren
    
    StartStoppApp a = new StartStoppApp ();
    
    a.setStartStoppKonfiguration( "C:\\JUnit");
    a.setStartStoppInkarnationsName( "StartStopp" );
    a.bestimmeRechnerAdresse();
    
    // Prozessverwaltung
    
    m_pv = ProzessVerwaltung.getInstanz();
  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
    m_pv.reset();
    
    // Skriptverwaltung anlegen

    String xmlAlsString = einlesenAusResource( m_resource );
    
    schreibeInDatei( m_datei, xmlAlsString );
    
    m_sv = new SkriptVerwaltung (m_datei, m_absender, 0);
    
    if (m_sv.isFehlerStartStopp())
      fail ( m_sv.getFehlerText() );
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.ProzessVerwaltung#getAnzahlProzesse()}.
   * Methode prft ob die Anzahl der eingelesenen Prozesse stimmt
   */
  @Test
  public void testGetAnzahlProzesse()
  {
    int anzahl = m_pv.getAnzahlProzesse();
    
    assertEquals( m_erwarteteNamen.size(), anzahl );
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.ProzessVerwaltung#getAlleProzessDaten()}.
   * Methode prft ob die Prozessdaten richtig eingelesen werden
   */
  @Test
  public void testGetAlleProzessDaten()
  {
    HashMap<String, Boolean> gefunden = new HashMap<String, Boolean> ();
    Iterator<String> it = m_erwarteteNamen.iterator();
    while (it.hasNext())
      gefunden.put( it.next(), false );
    
    List<ProzessDaten> pd = m_pv.getAlleProzessDaten();
    
    if (pd.size() != m_erwarteteNamen.size())
      fail ("Anzahl Prozesse stimmt nicht !");
    
    for (int i=0; i<pd.size(); ++i)
      if (gefunden.containsKey( pd.get( i ).getName()) )
        gefunden.put (pd.get( i ).getName(), true);
      else
        fail ("Falscher erwarteter Inkarnationsname: " + pd.get( i ).getName());  
        
    if (gefunden.containsValue( false ))
      fail ("Nicht alle Inkarnationen gefunden !");  

    assert (true);
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.ProzessVerwaltung#getProzessDaten(java.lang.String)}.
   */
  @Test
  public void testGetProzessDatenString()
  {
    String zuPruefendeApplikation = "Applikation1.1";
    
    String id = m_pv.getPidByName( zuPruefendeApplikation );
    if (id == null)
     fail( "Pid fr Applikation " + zuPruefendeApplikation + " nicht gefunden !" );
    
    ProzessDaten pd = m_pv.getProzessDaten( id );
    if (pd == null)
      fail( "Keine Prozessdaten fr Applikation " + zuPruefendeApplikation + " gefunden !" );
    
    // Ausfhrbare Datei
    
    if (!pd.getAusfuehrbareDatei().equals( "java -cp" ))
      fail( "Fehler bei ausfhrbarer Datei: " + pd.getAusfuehrbareDatei() );

    // Startart
    
    StartArt sa = pd.getStartArt();
    
    if (sa.getNeuStart() != E_NEUSTART.Nein )
      fail( "Fehler bei Startart - Neustart: " + sa.getNeuStart().getText() );

    if (sa.getOption() != E_STARTART.MANUELL )
      fail( "Fehler bei Startart - Option: " + sa.getOption().getText() );

    if (!sa.getIntervallZeit().equals( "* 2 * * *" ))
      fail( "Fehler bei Startart - Intervallzeit: " + sa.getIntervallZeit() );

    // Standardausgabe
    
    StandardAusgabe stda = pd.getStandardAusgabe();
    
    if (!stda.getDateiAlias().equals( "App1.1Out" ))
      fail( "Fehler bei Dateialias " + stda.getDateiAlias() );

    if (!stda.getDateiName().equals( "C:\\kernsoftware-startstopp-r1\\StartStopp\\app1_1_out.txt" ))
      fail( "Fehler bei Dateialias " + stda.getDateiName() );

    if (stda.getMaxGroesse() != 1)
      fail( "Fehler bei Dateigre " + stda.getMaxGroesse());

    if (stda.getOption() != E_AUSGABE.EIGENEDATEI)
      fail( "Fehler bei Dateioption " + stda.getOption().getText());
    
    // Standardfehlerausgabe
    
    FehlerAusgabe stdfa = pd.getStandardFehlerAusgabe();
    
    if (!stdfa.getDateiAlias().equals( "App1.1Err" ))
      fail( "Fehler bei Dateialias " + stda.getDateiAlias() );

    if (!stdfa.getDateiName().equals( "C:\\kernsoftware-startstopp-r1\\StartStopp\\app1_1_err.txt" ))
      fail( "Fehler bei Dateialias " + stda.getDateiName() );

    if (stdfa.getMaxGroesse() != 1)
      fail( "Fehler bei Dateigre " + stda.getMaxGroesse());

    if (stdfa.getOption() != E_AUSGABE.EIGENEDATEI)
      fail( "Fehler bei Dateioption " + stda.getOption().getText());
    
    // Startbedingung
    
    List<StartBedingung> startBedingungen = pd.getStartBedingung();
    if (startBedingungen.size() != 2)
      fail( "Falsche Anzahl Startbedingungen: " + startBedingungen.size());
    
    StartBedingung sb = startBedingungen.get( 0 );
    
    if (!sb.getProzessName().equals( "Applikation1.2" ))
      fail( "Fehler Startbedingung - Prozessname: " + sb.getProzessName());
    
    if (!sb.getRechnerAlias().equals( "" ))
      fail( "Fehler Startbedingung - Rechnername: " + sb.getRechnerAlias());
      
    if (sb.getWarteZeit() != 30)
      fail( "Fehler Startbedingung - Wartezeit: " + sb.getWarteZeit());
    
    
    assert (true);
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.ProzessVerwaltung#loescheProzess(java.lang.String, long)}.
   * Methode lscht einen Prozess
   */
  @Test
  public void testLoescheProzess()
  {
    String zuLoeschendeApplikation = "Applikation1.1";
    
    String id = m_pv.getPidByName( zuLoeschendeApplikation );
    if (id == null)
     fail( "Pid fr Applikation " + zuLoeschendeApplikation + " nicht gefunden !" );

    // Applikation lschen
    
    m_pv.loescheProzess( id, m_absender );
    
    mySleep( 5000 ); // wegen Thread der den Prozess lscht
    
    List<ProzessDaten> pd = m_pv.getAlleProzessDaten();
    Iterator<ProzessDaten> it = pd.iterator();
    while (it.hasNext())
    {
      ProzessDaten p = it.next();
      if (p.getName().equals( zuLoeschendeApplikation ))
        fail ("Applikation " + zuLoeschendeApplikation + " wurde nicht gelscht !");
    }
    
    assert (true);
  }

  /**
   * Methode zum Einlesen einer XML-Datei aus den Resourcen in einen String
   * @param resourceName Name der Resource
   * @return String mit XML-Struktur
   */
  public String einlesenAusResource (String resourceName)
  {
    String vorgefertigteXMLStruktur = null;
    
    InputStream is = this.getClass().getResourceAsStream( resourceName );
    if (is == null)
      return "";
    
    BufferedInputStream bis = new BufferedInputStream (is, 4096);
    StringBuffer sb = new StringBuffer ();
    
    byte [] buf = new byte [4096];
    
    try
    {
      for (int len; (len = bis.read(buf)) !=-1;)
      {
        sb.append( new String (buf, 0, len) );
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }

    vorgefertigteXMLStruktur = sb.toString();
    
    return vorgefertigteXMLStruktur;
  }

  /**
   * Methode schreibt einen String in eine Datei
   * @param dateiName Dateiname
   * @param text String der in die Datei geschrieben wird
   */
  public void schreibeInDatei (String dateiName, String text)
  {
    FileOutputStream schreibeStrom = null;
    
    try
    {
      schreibeStrom = new FileOutputStream(dateiName);
      
      for (int i=0; i < text.length(); i++)
        schreibeStrom.write((byte)text.charAt(i));

      schreibeStrom.close();
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
    }
    
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }
  
  /**
   * Sleep Methode
   * @param time Zeit in Millisekunden
   */
  private void mySleep (long time)
  {
    try
    {
      Thread.sleep( time );
    }
    catch (InterruptedException e)
    {
    }
  }

}
